__doc__='''ZCache'''
__version__='0.3'

from Globals import HTMLFile, MessageDialog, package_home
import OFS.SimpleItem, Acquisition, AccessControl.Role
from BoboPOS import Persistence
from DateTime import DateTime
import sys

def manage_addZCache(self, id, cacheObject, delay, REQUEST=None):
    """Add ZCache Object"""
    i=ZCache(id, cacheObject, delay, REQUEST)
    self._setObject(id,i)
    if REQUEST is not None:
	return self.manage_main(self,REQUEST)

manage_addZCacheForm=HTMLFile('zcacheAdd', globals())

class ZCache(
    OFS.SimpleItem.Item,
    Persistence.Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager,
    ):
	"""ZCache object - store self.cacheObject's results in memory, and return it when called, 
     updating the stored value at most every self.delay seconds"""
	meta_type = 'ZCache'

	icon = 'zcache'

	manage=manage_main=HTMLFile('manageZCache', globals())

	manage_options=({'icon':'', 'label':'Properties',
                    'action':'manage_main', 'target':'manage_main'},
				    {'icon':'', 'label':'View',
                    'action':'index_html', 'target':'manage_main'},
					{'icon':'', 'label':'Security',
                    'action':'manage_access', 'target':'manage_main'})
			 
	__ac_permissions__=(
	        ('View management screens', ('manage', 'manage_main', 'manage_access',)),
		('Manage properties', ('manage_makeChanges',)),
		('View', ('index_html',)),
		('Update Cache', ('update',)),
		)
					
    
	def __init__(self, id, cacheObject, delay, REQUEST):
		"""Create a ZCache - pass object to cache and minimum delay between reads (in seconds) """
		self.id	= id
		self.cacheObject = cacheObject
		self.lastreadtime = 0
		self.cache = None
		self.delay = delay
		self.size = -1
    
	def index_html(self, REQUEST, RESPONSE):
		"""Return contents of cache, updating if hasn't been updated for more that self.delay seconds"""
		from string import split
		from webdav.common import rfc1123_date
		now = DateTime().timeTime()
		ms=REQUEST.get_header('If-Modified-Since', None)
		if ms is not None:
			ms=split(ms, ';')[0]
			ms=DateTime(ms).timeTime()
			if self.lastreadtime > ms:
				RESPONSE.setStatus(304)
				return RESPONSE
																					
		if (self.cache is not None) and (now > self.lastreadtime + self.delay):
			self.cache = None
		if self.cache is None:
			self._update(REQUEST)
		RESPONSE.setHeader('Last-Modified', rfc1123_date(self.lastreadtime))
		RESPONSE.setHeader('Content-Length', self.size)
		RESPONSE.write(self.cache)
		return ''

	def __str__(self):
		""" return contents of cache when called by <dtml-var> """
		now = DateTime().timeTime()
		if (self.cache is not None) and (now > self.lastreadtime + self.delay):
			self.cache = None
		if self.cache is None:
			self._update(self.REQUEST)
		return self.cache
		
	def _update(self, REQUEST=None):
		""" Force update of contents of cache """
		cacheObject = self.aq_parent[self.cacheObject]
		if REQUEST['AUTHENTICATED_USER'].has_permission("View", cacheObject):
			try: newcache = cacheObject(self, REQUEST)
			except: raise sys.exc_type, sys.exc_value
			if newcache != self.cache:
				self.cache = newcache
				self.lastreadtime = DateTime().timeTime()
				self.size = len(self.cache)
		else:
			raise 'Unauthorized', ('You are not authorized to access the cached object')
	
	update = _update
	
	def manage_makeChanges(self, cacheObject, delay, REQUEST=None):
		""" Change properties """
		self.cacheObject = cacheObject
		self.delay = delay
		self._update(REQUEST)
		if REQUEST: return MessageDialog(title  ='Changed %s' % self.__name__,
					 message='%s has been updated' % self.id,
				     action =REQUEST['URL2']+'/manage_main',
					 target ='manage_main')